<?php
class Ninetheme_Widget_Product_Categories_Walker extends Walker {
    /**
	 * What the class handles.
	 *
	 * @var string
	 */
	public $tree_type = 'product_cat';

	/**
	 * DB fields to use.
	 *
	 * @var array
	 */
	public $db_fields = array(
		'parent' => 'parent',
		'id'     => 'term_id',
		'slug'   => 'slug',
	);
	/**
	 * Starts the list before the elements are added.
	 *
	 * @see Walker::start_lvl()
	 * @since 2.1.0
	 *
	 * @param string $output Passed by reference. Used to append additional content.
	 * @param int    $depth Depth of category. Used for tab indentation.
	 * @param array  $args Will only append content if style argument value is 'list'.
	 */
	public function start_lvl( &$output, $depth = 0, $args = array() ) {
		if ( 'list' !== $args['style'] ) {
			return;
		}

		$indent  = str_repeat( "\t", $depth );
		$plus  = ninetheme_svg_lists('plus');
		$output .= "$indent<span class='subDropdown pluss'>$plus</span><ul class='children'>\n";
	}

	/**
	 * Ends the list of after the elements are added.
	 *
	 * @see Walker::end_lvl()
	 * @since 2.1.0
	 *
	 * @param string $output Passed by reference. Used to append additional content.
	 * @param int    $depth Depth of category. Used for tab indentation.
	 * @param array  $args Will only append content if style argument value is 'list'.
	 */
	public function end_lvl( &$output, $depth = 0, $args = array() ) {
		if ( 'list' !== $args['style'] ) {
			return;
		}

		$indent  = str_repeat( "\t", $depth );
		$output .= "$indent</ul>\n";
	}

	/**
	 * Start the element output.
	 *
	 * @see Walker::start_el()
	 * @since 2.1.0
	 *
	 * @param string  $output            Passed by reference. Used to append additional content.
	 * @param object  $cat               Category.
	 * @param int     $depth             Depth of category in reference to parents.
	 * @param array   $args              Arguments.
	 * @param integer $current_object_id Current object ID.
	 */
	public function start_el( &$output, $cat, $depth = 0, $args = array(), $current_object_id = 0 ) {
	    
		$tax = $args['tax_type'];
		$cat_id = intval( $cat->term_id );
        
		$output .= '<li class="'.$tax.'-item '.$tax.'-item-' . $cat_id;

		if ( $args['current_category'] === $cat_id ) {
			$output .= ' current-'.$tax;
		}

		if ( $args['has_children'] && $args['hierarchical'] && ( empty( $args['max_depth'] ) || $args['max_depth'] > $depth + 1 ) ) {
			$output .= ' '.$tax.'-parent';
		}

		if ( isset( $args['current_category_ancestors'] ) && $args['current_category'] && in_array( $cat_id, $args['current_category_ancestors'], true ) ) {
			$output .= ' current-'.$tax.'-parent';
		}
		$checkbox = '';
		    
        if ( isset( $_GET['filter_'.$tax ] ) ) {
            if ( in_array( $cat_id, explode( ',', $_GET['filter_'.$tax ] ) ) ) {
                $checkbox = ' checked';
                $output .= ' checked';
            }
        }
		$output .= '"><a class="product_'.$tax.' '.esc_attr( $checkbox ).'" href="'.esc_url( ninetheme_wc_get_tax_url( $cat_id, $tax ) ).'">';
        $output .= $cat->name;
        $output .= '</a>';
        $output .= true == $args['count'] && 'cat' != $tax ? '<span class="widget-list-span">'.$cat->count.'</span>' : '';
	}

	/**
	 * Ends the element output, if needed.
	 *
	 * @see Walker::end_el()
	 * @since 2.1.0
	 *
	 * @param string $output Passed by reference. Used to append additional content.
	 * @param object $cat    Category.
	 * @param int    $depth  Depth of category. Not used.
	 * @param array  $args   Only uses 'list' for whether should append to output.
	 */
	public function end_el( &$output, $cat, $depth = 0, $args = array() ) {
		$output .= "</li>\n";
	}

	/**
	 * Traverse elements to create list from elements.
	 *
	 * Display one element if the element doesn't have any children otherwise,
	 * display the element and its children. Will only traverse up to the max.
	 * depth and no ignore elements under that depth. It is possible to set the.
	 * max depth to include all depths, see walk() method.
	 *
	 * This method shouldn't be called directly, use the walk() method instead.
	 *
	 * @since 2.5.0
	 *
	 * @param object $element           Data object.
	 * @param array  $children_elements List of elements to continue traversing.
	 * @param int    $max_depth         Max depth to traverse.
	 * @param int    $depth             Depth of current element.
	 * @param array  $args              Arguments.
	 * @param string $output            Passed by reference. Used to append additional content.
	 * @return null Null on failure with no changes to parameters.
	 */
	public function display_element( $element, &$children_elements, $max_depth, $depth, $args, &$output ) {
		if ( ! $element || ( 0 === $element->count && ! empty( $args[0]['hide_empty'] ) ) ) {
			return;
		}
		parent::display_element( $element, $children_elements, $max_depth, $depth, $args, $output );
	}
}

class Ninetheme_Widget_Product_Categories extends WP_Widget {

    // Widget Settings
    function __construct() {
        $widget_ops = array('description' => esc_html__('For Main Shop Page.','crisop') );
        $control_ops = array( 'width' => 300, 'height' => 350, 'id_base' => 'ninetheme_product_categories' );
        parent::__construct( 'ninetheme_product_categories', esc_html__('Crisop Product Categories','crisop'), $widget_ops, $control_ops );
    }

    // Widget Output
    function widget( $args, $instance )
    {
        extract( $args );
        if ( class_exists( '\Elementor\Plugin' ) && \Elementor\Plugin::$instance->editor->is_edit_mode() ) {
            
            $title      = apply_filters( 'widget_title', empty($instance['title'] ) ? '' : $instance['title'], $instance );
            $exclude    = $instance['exclude'];
            $hide_empty = $instance['hide_empty'];

            echo $before_widget;

            if ( $title ) {
                echo $before_title . $title . $after_title;
            }

            wp_enqueue_script( 'ninetheme-widget-product-categories' );
            wp_enqueue_style( 'ninetheme-widget-product-categories' );

            echo '<div class="ninetheme-widget-product-categories widget-body site-checkbox-lists">';
                echo '<div class="site-scroll">';
                    echo '<ul class="ninetheme-product-categories-primary ninetheme-scrollbar">';
        
                        echo wp_list_categories(array(
                            'echo'       => true,
                            'taxonomy'   => 'product_cat',
                            'tax_type'   => 'cat',
                            'count'      => true,
                            'depth'      => 5,
                            'hide_empty' => 0,
                            'title_li'   => '',
                            'exclude'    => !empty( $exclude ) ? $exclude : '',
                            'walker'     => new Ninetheme_Widget_Product_Categories_Walker()
                        ));
                        
                    echo '</ul>';
                echo '</div>';
            echo '</div>';

            echo $after_widget;
                
        } else {
        
            if ( is_shop() && !is_product_category() )
            {
                $title      = apply_filters( 'widget_title', empty($instance['title'] ) ? '' : $instance['title'], $instance );
                $exclude    = $instance['exclude'];
                $hide_empty = $instance['hide_empty'];
    
                echo $before_widget;
    
                if ( $title ) {
                    echo $before_title . $title . $after_title;
                }
    
                wp_enqueue_script( 'ninetheme-widget-product-categories' );
                wp_enqueue_style( 'ninetheme-widget-product-categories' );
    
                echo '<div class="ninetheme-widget-product-categories widget-body site-checkbox-lists">';
                    echo '<div class="site-scroll">';
                        echo '<ul class="ninetheme-product-categories-primary ninetheme-scrollbar">';
            
                            echo wp_list_categories(array(
                                'echo'       => true,
                                'taxonomy'   => 'product_cat',
                                'tax_type'   => 'cat',
                                'count'      => true,
                                'depth'      => 5,
                                'hide_empty' => 0,
                                'title_li'   => '',
                                'exclude'    => !empty( $exclude ) ? $exclude : '',
                                'walker'     => new Ninetheme_Widget_Product_Categories_Walker()
                            ));
                            
                        echo '</ul>';
                    echo '</div>';
                echo '</div>';
    
                echo $after_widget;
            }
        }
    }

    // Update
    function update( $new_instance, $old_instance )
    {
        $instance = $old_instance;

        $instance['title']      = strip_tags($new_instance['title']);
        $instance['hide_empty'] = !empty( $new_instance['hide_empty'] ) ? $new_instance['hide_empty'] : '';
        $instance['exclude']    = $new_instance['exclude'];

        return $instance;
    }

    // Backend Form
    function form( $instance )
    {
        $defaults   = array( 'title' => 'Product Categories', 'exclude' => array(), 'hide_empty' => '' );
        $instance   = wp_parse_args(( array ) $instance, $defaults );
        $select     = is_array( $instance['exclude'] ) ? $instance['exclude'] : array();
        $hide_empty = $instance['hide_empty'] ? 1 : 0;

        $terms = get_terms( array(
            'taxonomy'   => 'product_cat',
            'hide_empty' => $hide_empty,
            'parent'     => 0
        ));
        wp_enqueue_style( 'select2-full' );
        wp_enqueue_script( 'select2-full' );
        ?>
        <p>
            <label for="<?php echo $this->get_field_id('title'); ?>"><?php esc_html_e('Title:','crisop'); ?></label>
            <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" value="<?php echo $instance['title']; ?>" />
        </p>
        <p>
            <label for="<?php echo $this->get_field_id('hide_empty'); ?>"><?php esc_html_e( 'Hide if empty:','crisop' ); ?></label>
            <input type="checkbox" class="widefat" id="<?php echo $this->get_field_id('hide_empty'); ?>" name="<?php echo $this->get_field_name('hide_empty'); ?>" value="1" <?php checked( $instance['hide_empty'], 1 ); ?> />
        </p>
        <p>
            <label for="<?php echo $this->get_field_id('exclude'); ?>"><?php esc_html_e( 'Exclude Category(s):','crisop' ); ?></label>
            <select class="ninetheme-select2" id="<?php echo $this->get_field_id('exclude'); ?>" name="<?php echo $this->get_field_name('exclude'); ?>[]" multiple="multiple">
                <?php foreach ( $terms as $term ) {
                    $selected = in_array( $term->term_id, $select) ? 'selected="selected"' : '';
                    ?>
                    <option value="<?php echo $term->term_id; ?>" <?php echo $selected; ?>><?php echo $term->name; ?></option>
                <?php } ?>
            </select>
        </p>
        <?php
    }
}

// Add Widget
function ninetheme_widget_product_categories_init() {
    register_widget('Ninetheme_Widget_Product_Categories');
}
add_action('widgets_init', 'ninetheme_widget_product_categories_init');

?>
